"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SynchronizationConfiguration = void 0;
const rxjs_1 = require("rxjs");
const base_model_1 = require("../utils/base.model");
class SynchronizationConfiguration extends base_model_1.BaseModel {
    constructor(...parts) {
        super({ skipZoneUpdate: false }, ...parts);
        this.BASE_TYPE = SynchronizationConfiguration;
        this.keys = [];
        this.refreshFunctions = [];
        this.refreshLocalValueFunctions = [];
        this.clearFunctions = [];
        this._valueChanged = new rxjs_1.Subject();
        this.valueChanged = this._valueChanged.asObservable();
    }
    GetDefaultValue() {
        return { skipZoneUpdate: false, storage: localStorage, keys: [], takeAlwaysFromStorage: false };
    }
    notify() {
        if (SynchronizationConfiguration && SynchronizationConfiguration.ngZoneRun && !this.skipZoneUpdate) {
            SynchronizationConfiguration.ngZoneRun(() => {
                this._valueChanged.next();
            });
        }
        else {
            this._valueChanged.next();
        }
    }
    refresh() {
        if (this.takeAlwaysFromStorage) {
            throw new Error('"refresh" function can only be used when "takeAlwaysFromStorage" is false');
        }
        this.refreshFunctions.forEach((f) => f());
        this.notify();
    }
    refreshLocalValue() {
        if (this.takeAlwaysFromStorage) {
            throw new Error('"refreshLocalValue" function can only be used when "takeAlwaysFromStorage" is false');
        }
        this.refreshLocalValueFunctions.forEach((f) => f());
        this.notify();
    }
    clear() {
        this.clearFunctions.forEach((f) => f());
        this.notify();
    }
}
exports.SynchronizationConfiguration = SynchronizationConfiguration;
//# sourceMappingURL=SynchronizationConfiguration.js.map