"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Synchronize = void 0;
const SynchronizationConfiguration_1 = require("./SynchronizationConfiguration");
function Synchronize(config, create, save, runOnInit) {
    return function (target, propertyName) {
        const ownDescriptor = Object.getOwnPropertyDescriptor(target, propertyName);
        let localValue = undefined;
        const key = `${config.prefix}.${propertyName}`;
        config.keys.push(key);
        function getFromStorage() {
            let value = config.storage.getItem(key);
            value = value !== null ? JSON.parse(value) : value;
            if (create && value) {
                value = create(value);
            }
            return value;
        }
        const symbol = Symbol("eventValue");
        function getItem(eventValue = symbol) {
            if (!config.takeAlwaysFromStorage &&
                localValue &&
                eventValue === symbol) {
                return localValue;
            }
            else {
                let value = eventValue !== symbol ? eventValue : config.storage.getItem(key);
                value = value !== null ? JSON.parse(value) : value;
                if (create && value) {
                    value = create(value);
                }
                if (value !== localValue) {
                    localValue = value;
                    config.notify();
                }
                return localValue;
            }
        }
        localValue = getFromStorage();
        window.addEventListener("storage", (e) => {
            if (e.key === key) {
                localValue = getItem(e.newValue);
                config.notify();
                if (config.onStorageEvent) {
                    config.onStorageEvent(e);
                }
                if (SynchronizationConfiguration_1.SynchronizationConfiguration.ON_STORAGE_EVENT) {
                    SynchronizationConfiguration_1.SynchronizationConfiguration.ON_STORAGE_EVENT(e);
                }
            }
        });
        function setItem(value) {
            localValue = value;
            if (value === undefined) {
                config.storage.removeItem(key);
            }
            else {
                config.storage.setItem(key, JSON.stringify(save ? save(localValue) : localValue));
            }
            config.notify();
        }
        config.refreshFunctions.push(() => {
            setItem(localValue);
        });
        config.refreshLocalValueFunctions.push(() => {
            localValue = getFromStorage();
        });
        config.clearFunctions.push(() => {
            config.storage.removeItem(key);
            localValue = undefined;
        });
        const descriptor = {
            configurable: true,
            get: () => {
                return getItem();
            },
            set: (v) => {
                setItem(v);
            },
        };
        Object.defineProperty(target, propertyName, descriptor);
        if (runOnInit) {
            runOnInit();
        }
    };
}
exports.Synchronize = Synchronize;
//# sourceMappingURL=synchronize.decorator.js.map