import 'reflect-metadata';
export declare type OmitingBaseModelField<T extends string | number | symbol = ''> = T | 'ToFormData' | 'ToPlainObject' | 'InitializeProperty' | 'InitializeArrayProperty' | 'InitializeEnumProperties' | 'InitializeProperties' | 'InitializeArrayProperties' | 'ToStringEnumsModel' | 'ToStringPropertiesModel' | 'StringifyProperties' | 'ToCreateModel' | 'ToUpdateModel' | 'BASE_TYPE' | 'GetDefaultValue' | 'Clone';
export declare type BaseModelPart<TModel, TOmitingTypes extends keyof TModel = never> = Partial<Omit<TModel, OmitingBaseModelField<TOmitingTypes>>>;
export declare type StrictBaseModelPart<TModel, TOmitingTypes extends keyof TModel = never> = Omit<TModel, OmitingBaseModelField<TOmitingTypes>>;
export interface IBaseModelConstructor<TModel, TOmitingTypes extends keyof TModel = never> {
    new (...parts: BaseModelPart<TModel, TOmitingTypes>[]): TModel;
}
declare const ENUM_PROPERTIES: unique symbol;
declare const INIT_PROPERTIES: unique symbol;
export declare type KeyOfType<T, U> = {
    [P in keyof T]: T[P] extends U ? P : never;
}[keyof T];
export declare type Flatten<T> = T extends Array<infer R> ? R : T;
export declare type BaseModelSafeKeys<TModel, TOmitingTypes extends keyof TModel = never> = keyof Omit<TModel, OmitingBaseModelField<TOmitingTypes>>;
export declare type BaseModelSafeArrayKeys<TModel, TOmitingTypes extends keyof TModel = never> = KeyOfType<Omit<TModel, OmitingBaseModelField<TOmitingTypes>>, any[]>;
export declare abstract class BaseModel<TModel, TOmitingTypes extends keyof TModel = never> {
    protected abstract BASE_TYPE: IBaseModelConstructor<TModel, TOmitingTypes>;
    abstract GetDefaultValue(): StrictBaseModelPart<TModel, TOmitingTypes>;
    constructor(...parts: BaseModelPart<TModel, TOmitingTypes>[]);
    ToFormData(fields?: (keyof this)[] | null): FormData;
    ToPlainObject(): Record<Exclude<keyof TModel, OmitingBaseModelField<TOmitingTypes>>, any>;
    private InitializeProperties;
    private CreateItem2;
    private readonly [ENUM_PROPERTIES]?;
    private readonly [INIT_PROPERTIES]?;
    private InitializeEnumProperties;
    ToStringEnumsModel(): TModel;
    Clone(...parts: BaseModelPart<TModel, TOmitingTypes>[]): TModel;
}
export declare function createTypeMap(typeMap: [(obj: any) => boolean, IBaseModelConstructor<any> | null][]): Map<(obj: any) => boolean, IBaseModelConstructor<any> | null>;
export declare function Property(ctr: () => IBaseModelConstructor<any>): PropertyDecorator;
export declare function Property(ctr: () => Map<(obj: any) => boolean, IBaseModelConstructor<any> | null>): PropertyDecorator;
export declare function EnumProperty(enumPropertyFactory: () => any): (target: any, propertyName: string | symbol) => void;
export declare const NotMapped: () => (target: any, propertyName: string) => void;
export {};
//# sourceMappingURL=base.model.d.ts.map