"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NotMapped = exports.EnumProperty = exports.Property = exports.createTypeMap = exports.BaseModel = void 0;
require("reflect-metadata");
const ENUM_PROPERTIES = Symbol('ENUM_PROPERTIES');
const NOT_MAPPED_PROPERTIES = Symbol('NOT_MAPPED_PROPERTIES');
const INIT_PROPERTIES = Symbol('INIT_PROPERTIES');
const INIT_ARRAY_PROPERTIES = Symbol('INIT_ARRAY_PROPERTIES');
class BaseModel {
    constructor(...parts) {
        const defaultValue = this.GetDefaultValue();
        Object.assign(this, defaultValue, ...parts);
        this.InitializeEnumProperties();
        this.InitializeProperties();
    }
    ToFormData(fields = null) {
        if (fields == null) {
            fields = Object.keys(this).filter((field) => field != 'BASE_TYPE');
        }
        const formData = new FormData();
        fields.forEach((field) => {
            const fieldValue = this[field];
            if (fieldValue instanceof File) {
                formData.append(field, fieldValue, fieldValue.name);
            }
            else if (fieldValue instanceof Date) {
                formData.append(field, fieldValue.toISOString());
            }
            else {
                formData.append(field, `${fieldValue}`);
            }
        });
        return formData;
    }
    ToPlainObject() {
        const keys = Object.keys(this);
        const obj = {};
        keys.forEach((key) => {
            var value = this[key];
            if (typeof value != 'function') {
                if (value instanceof BaseModel) {
                    obj[key] = value.ToPlainObject();
                }
                else {
                    obj[key] = value;
                }
            }
        });
        return obj;
    }
    InitializeProperties() {
        var _a;
        (_a = this[INIT_PROPERTIES]) === null || _a === void 0 ? void 0 : _a.forEach((prop) => {
            const ctrFactory = Reflect.getMetadata('design:initialize-property', this, prop);
            this[prop] = this.CreateItem2(this[prop], ctrFactory());
        });
    }
    CreateItem2(item, ctrOrMap) {
        var _a;
        if (item == null) {
            return item;
        }
        if (Array.isArray(item)) {
            return item.map((i) => this.CreateItem2(i, ctrOrMap));
        }
        else {
            let ctr = ctrOrMap;
            if (ctrOrMap instanceof Map) {
                const ctrs = Array.from(ctrOrMap);
                ctr = (_a = ctrs.find((i) => i[0](item))) === null || _a === void 0 ? void 0 : _a[1];
            }
            if (ctr) {
                if (item instanceof ctr) {
                    return item;
                }
                else {
                    return new ctr(item);
                }
            }
            else {
                return item;
            }
        }
    }
    InitializeEnumProperties() {
        const enums = this[ENUM_PROPERTIES];
        if (!enums) {
            return;
        }
        enums.forEach((enumProperty) => {
            if (this[enumProperty.propertyName] != null && typeof this[enumProperty.propertyName] === 'string') {
                const enumType = enumProperty.enumPropertyFactory();
                this[enumProperty.propertyName] = enumType[this[enumProperty.propertyName]];
            }
        });
    }
    ToStringEnumsModel() {
        const model = new this.BASE_TYPE(this);
        const enums = this[ENUM_PROPERTIES];
        if (!enums) {
            return model;
        }
        enums.forEach((enumProperty) => {
            if (model[enumProperty.propertyName] != null) {
                const enumType = enumProperty.enumPropertyFactory();
                model[enumProperty.propertyName] = enumType[model[enumProperty.propertyName]];
            }
        });
        return model;
    }
    Clone(...parts) {
        const model = new this.BASE_TYPE(this.ToPlainObject(), ...parts);
        return model;
    }
}
exports.BaseModel = BaseModel;
function createTypeMap(typeMap) {
    return new Map(typeMap);
}
exports.createTypeMap = createTypeMap;
function Property(ctr) {
    return (target, propertyName) => {
        if (!target[INIT_PROPERTIES]) {
            target[INIT_PROPERTIES] = [];
        }
        target[INIT_PROPERTIES].push(propertyName);
        Reflect.metadata('design:initialize-property', ctr)(target, propertyName);
    };
}
exports.Property = Property;
function EnumProperty(enumPropertyFactory) {
    return (target, propertyName) => {
        if (!target[ENUM_PROPERTIES]) {
            target[ENUM_PROPERTIES] = [];
        }
        target[ENUM_PROPERTIES].push({
            propertyName,
            enumPropertyFactory,
        });
    };
}
exports.EnumProperty = EnumProperty;
const NotMapped = () => {
    return (target, propertyName) => {
        if (!target[NOT_MAPPED_PROPERTIES]) {
            target[NOT_MAPPED_PROPERTIES] = [];
        }
        target[NOT_MAPPED_PROPERTIES].push(propertyName);
    };
};
exports.NotMapped = NotMapped;
//# sourceMappingURL=base.model.js.map