"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base64 = void 0;
const tslib_1 = require("tslib");
const js_base64_1 = require("js-base64");
class Base64 {
    static toBase64(obj) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (obj instanceof Blob) {
                return yield Base64._read(obj);
            }
            else if (obj instanceof ArrayBuffer) {
                const blob = new Blob([obj], { type: 'application/octet-binary' });
                return yield Base64._read(blob);
            }
            else if (typeof obj === 'string') {
                return js_base64_1.Base64.encode(obj);
            }
            else {
                return js_base64_1.Base64.encode(JSON.stringify(obj));
            }
        });
    }
    static decode(base64str, returnType = 'string', contentType) {
        switch (returnType) {
            case 'string':
                return js_base64_1.Base64.decode(base64str);
            case 'object':
                return JSON.parse(js_base64_1.Base64.decode(base64str));
            case 'ArrayBuffer':
                const str2ab = js_base64_1.Base64.atob(base64str);
                return Base64.str2ab(str2ab);
            case 'Blob':
                const str2blob = js_base64_1.Base64.atob(base64str);
                const bytes = Base64.str2ab(str2blob);
                return new Blob([bytes]);
        }
    }
    static str2ab(str) {
        var buf = new ArrayBuffer(str.length * 2);
        var bufView = new Uint16Array(buf);
        for (var i = 0, strLen = str.length; i < strLen; i++) {
            bufView[i] = str.charCodeAt(i);
        }
        return buf;
    }
    static _read(blob) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.onloadend = () => {
                resolve(reader.result);
            };
            reader.onerror = () => {
                reject(reader.error);
            };
            reader.readAsDataURL(blob);
        });
    }
}
exports.Base64 = Base64;
//# sourceMappingURL=base64.js.map