"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperationCanceled = exports.CancelationToken = exports.CancelationTokenStore = void 0;
class CancelationTokenStore {
    constructor() { }
    createToken(parentToken) {
        const token = new CancelationToken({ store: this, parentToken });
        if (parentToken) {
            parentToken.childTokens.push(token);
        }
        return token;
    }
    cancel(token) {
        token.isCanceled = true;
        token.childTokens.forEach(token => token.cancel());
    }
}
exports.CancelationTokenStore = CancelationTokenStore;
class CancelationToken {
    constructor(...parts) {
        Object.assign(this, CancelationToken.DEFAULTS, ...parts);
        if (!this.store) {
            throw new Error('CancelationTokenStore was not provided');
        }
    }
    throwIfCanceled(errorMessage = this.cancelationMessage) {
        if (this.isCanceled) {
            throw this.getCancelationError(errorMessage);
        }
    }
    getCancelationError(errorMessage = this.cancelationMessage) {
        return new OperationCanceled(errorMessage);
    }
    cancel() {
        this.store && this.store.cancel(this);
    }
}
exports.CancelationToken = CancelationToken;
CancelationToken.DEFAULTS = {
    cancelationMessage: 'Operation canceled',
    childTokens: [],
    isCanceled: true
};
class OperationCanceled extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, OperationCanceled.prototype);
    }
}
exports.OperationCanceled = OperationCanceled;
//# sourceMappingURL=cancelation-token.js.map