"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InMemoryStorage = void 0;
class InMemoryStorage {
    constructor() {
        this._items = [];
    }
    get length() {
        return this._items.length;
    }
    clear() {
        this._items.forEach(item => delete this[item.key]);
        this._items = [];
    }
    getItem(key) {
        return this[key] || null;
    }
    key(index) {
        return this._items[index].key;
    }
    removeItem(key) {
        const itemIndex = this._items.findIndex(i => i.key === key);
        if (itemIndex < 0) {
            return;
        }
        delete this[key];
        this._items.splice(itemIndex, 1);
    }
    setItem(key, value) {
        this[key] = value;
        const existingIndex = this._items.findIndex(i => i.key === key);
        if (existingIndex >= 0) {
            this._items.splice(existingIndex, 1);
        }
        this._items.push({ key, value });
    }
}
exports.InMemoryStorage = InMemoryStorage;
InMemoryStorage.instance = new InMemoryStorage();
//# sourceMappingURL=in-memory.storage.js.map